import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

public class OneToManyTest {
	static EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:test.tmp;drop");

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		EntityManager em = emf.createEntityManager();

		System.out.println("Building stores...");
		em.getTransaction().begin();
		for (int i = 0; i < 10; i++) {
			em.persist(new Store("Store " + i));
		}
		em.getTransaction().commit();

		em.getTransaction().begin();
		System.out.println("Building products...");
		for (int i = 0; i < 100; i++) {
			Product p = new Product("Product " + i);
			em.persist(p);
			long storeID = (i % 10) + 1;
			Store s = em.find(Store.class, storeID);
			// p.listOfStores.add(s);
			s.products.add(p);
		}
		em.getTransaction().commit();

		listStoresProducts();

		em.getTransaction().begin();
		System.out.println("Generating offers...");
		TypedQuery<Store> storeQuery = em.createQuery("select s from Store s", Store.class);

		for (Store s : storeQuery.getResultList()) {
			for (Product p : s.getProducts()) {
				Offer o = new Offer(p, s, "Offer for product: " + p.getId() + ", store: " + s.getId());
				em.persist(o);
			}
		}
		em.getTransaction().commit();

		System.out.println("Removing product id: 54");

		em.getTransaction().begin();
		Product p = em.find(Product.class, 54);
		em.remove(p);
		em.getTransaction().commit();
		em.flush();

		listOffersForStores();

		em.close();
		emf.close();

	}

	public static void listOffersForStores() {
		EntityManager em = emf.createEntityManager();
		System.out.println("Listing stores / offers:");
		TypedQuery<Store> query = em.createQuery("select store from Store store", Store.class);
		for (Store s : query.getResultList()) {
			System.out.println("Store: " + s.getId() + ", size: " + s.getOffers().size());
			for (Offer o : s.getOffers()) {
				System.out.println("\tOffer: " + o.getOfferName());
			}
		}
		em.close();

	}

	public static void listStoresProducts() {
		EntityManager em = emf.createEntityManager();
		System.out.println("Listing stores / products:");
		TypedQuery<Store> query = em.createQuery("select store from Store store", Store.class);
		for (Store s : query.getResultList()) {
			System.out.println("Store: " + s.getId());
			for (Product p : s.getProducts()) {
				System.out.println("\tProduct: " + p.getId() + ", product name: " + p.getProductname());
			}
		}
		em.close();
	}

	@Entity
	public static class Store {
		@Id
		@GeneratedValue(strategy = GenerationType.AUTO)
		private long id;
		@OneToMany(targetEntity = Offer.class, mappedBy = "store")
		private List<Offer> offers;
		private String storeName;
		@ManyToMany(targetEntity = Product.class, fetch = FetchType.LAZY)
		private Set<Product> products;

		public Store(String storeName) {
			super();
			this.storeName = storeName;
			this.products = new HashSet<Product>();
		}

		public long getId() {
			return id;
		}

		public List<Offer> getOffers() {
			return offers;
		}

		public void setOffers(List<Offer> offers) {
			this.offers = offers;
		}

		public String getStoreName() {
			return storeName;
		}

		public void setStoreName(String storeName) {
			this.storeName = storeName;
		}

		public Set<Product> getProducts() {
			return products;
		}

		public void setProducts(Set<Product> products) {
			this.products = products;
		}

	}

	@Entity
	public static class Product {
		@Id
		@GeneratedValue(strategy = GenerationType.AUTO)
		private long id;
		@OneToMany(targetEntity = Offer.class, mappedBy = "product", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
		private Set<Offer> offers;
		private String productname;
		@ManyToMany(targetEntity = Store.class, fetch = FetchType.LAZY, mappedBy = "products")
		private Set<Store> listOfStores;

		public Product(String productname) {
			super();
			this.productname = productname;
			// this.listOfStores = new HashSet();
		}

		public long getId() {
			return id;
		}

		public Set<Offer> getOffers() {
			return offers;
		}

		public void setOffers(Set<Offer> offers) {
			this.offers = offers;
		}

		public String getProductname() {
			return productname;
		}

		public void setProductname(String productname) {
			this.productname = productname;
		}

		public Set<Store> getListOfStores() {
			return listOfStores;
		}

		public void setListOfStores(Set<Store> listOfStores) {
			this.listOfStores = listOfStores;
		}

	}

	@Entity
	public static class Offer {
		@ManyToOne
		private Product product;
		@ManyToOne
		private Store store;
		private String offerName;

		public Offer(Product product, Store store, String offerName) {
			super();
			this.product = product;
			this.store = store;
			this.offerName = offerName;
		}

		public Product getProduct() {
			return product;
		}

		public void setProduct(Product product) {
			this.product = product;
		}

		public Store getStore() {
			return store;
		}

		public void setStore(Store store) {
			this.store = store;
		}

		public String getOfferName() {
			return offerName;
		}

		public void setOfferName(String offerName) {
			this.offerName = offerName;
		}
	}
}
